/* $Id: gldriver.c,v 1.2 1997/11/05 21:38:11 wtcorrea Exp wtcorrea $ */

#include <GL/gl.h>
#include <GL/glu.h>
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "app.h"
#include "float.h"
#include "gldriver.h"
#include "macros.h"

#define NDEBUG
#include "debug.h"

static unsigned int light;

void gl_traversal_start(void)
{
    glEnable(GL_DEPTH_TEST);

    light = GL_LIGHT0;
    glEnable(GL_LIGHTING);
    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);

    switch (app.view) {
    case VIEW_POINTS:
	DEBUG(fprintf(stderr, "View points not implemented for GL mode.\n"));
	DEBUG(fprintf(stderr, "Using view flat instead.\n"));
	glShadeModel(GL_FLAT);
	break;
    case VIEW_WIREFRAME:
	DEBUG(fprintf(stderr, "View wireframe not implemented for GL mode.\n"));
	DEBUG(fprintf(stderr, "Using view flat instead.\n"));
	glShadeModel(GL_FLAT);
	break;
    case VIEW_FLAT:
	glShadeModel(GL_FLAT);
	break;
    case VIEW_GOURAUD:
	glShadeModel(GL_SMOOTH);
	break;
    case VIEW_PHONG:
	DEBUG(fprintf(stderr, "View Phong not implemented for GL mode.\n"));
	DEBUG(fprintf(stderr, "Using view Gouraud instead.\n"));
	glShadeModel(GL_SMOOTH);
	break;
    }
}

void gl_traversal_end(void)
{
    /* nothing for now */
}

void gl_separator_start(void)
{
    glPushMatrix();
    glPushAttrib(GL_CURRENT_BIT);
}

void gl_separator_end(void)
{
    glPopAttrib();
    glPopMatrix();
}

void gl_camera(
     float *position,		/* x,y,z */
     float *orientation,	/* x,y,z,theta */
     float near_distance,
     float far_distance,
     float height_angle)
{
    glRotatef((float) -rtod(orientation[3]),
	orientation[0], orientation[1], orientation[2]);
    glTranslatef(-position[0], -position[1], -position[2]);

    switch (app.projection) {
    case PROJ_PERSPECTIVE:
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(
	    (double) rtod(height_angle),
	    (double) app.w / (double) app.h,
	    (double) near_distance, (double) far_distance);
	glMatrixMode(GL_MODELVIEW);
	break;
    case PROJ_ORTHOGONAL:
        fprintf(stderr, "Orthognal projection not implemented for GL mode\n");
        fprintf(stderr, "Using perspective instead.\n");
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(
	    (double) rtod(height_angle),
	    (double) app.w / (double) app.h,
	    (double) near_distance, (double) far_distance);
	glMatrixMode(GL_MODELVIEW);
	break;
    default:
	fprintf(stderr, "invalid kind of projection\n");
	exit(EXIT_FAILURE);
    }
}

void gl_point_light(
     float intensity,
     float *color,		/* r,g,b */
     float *location)
{				/* x,y,z */
    float black[] = {(float) 0.0, (float) 0.0, (float) 0.0, (float) 1.0};

    assert((light >= GL_LIGHT0) && (light <= GL_LIGHT7));
    color[0] *= intensity, color[1] *= intensity, color[2] *= intensity;
    glLightfv(light, GL_AMBIENT, color);
    glLightfv(light, GL_DIFFUSE, color);
    glLightfv(light, GL_SPECULAR, color);
    glLightfv(light, GL_POSITION, location);
    glEnable(light);
    light++;
}

void gl_material(
     float *ambient_color,	/* r,g,b */
     float *diffuse_color,	/* r,g,b */
     float *specular_color,	/* r,g,b */
     float *emissive_color,	/* r,g,b */
     float shininess)
{
    glMaterialfv(GL_FRONT, GL_AMBIENT, ambient_color);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse_color);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular_color);
    glMaterialfv(GL_FRONT, GL_EMISSION, emissive_color);
    glMaterialf(GL_FRONT, GL_SHININESS, shininess);
}

void gl_scale(float x, float y, float z)
{
    glScalef(x, y, z);
}

void gl_translation(float x, float y, float z)
{
    glTranslatef(x, y, z);
}

void gl_rotation(float x, float y, float z, float theta)
{
    glRotatef((float) rtod(theta), x, y, z);
}

void gl_label(char *label)
{
    (void) label;
}

float v3len(float *v)
{
    return sqrtf(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
}

void gl_triangle(
     float *v0,			/* x,y,z */
     float *v1,
     float *v2,
     float *n0,			/* normal for v0 */
     float *n1,
     float *n2)
{
    glBegin(GL_POLYGON);
    glNormal3fv(n0);
    glVertex3fv(v0);
    glNormal3fv(n1);
    glVertex3fv(v1);
    glNormal3fv(n2);
    glVertex3fv(v2);
    glEnd();
}

void gl_cube(float width, float height, float depth)
{
    (void) (width, height, depth);
}

void gl_sphere(float radius)
{
    (void) (radius);
}

void gl_cone(float bottom_radius, float height)
{
    (void) (bottom_radius, height);
}

void gl_cylinder(float radius, float height)
{
    (void) (radius, height);
}
